#include <cyu3system.h>
#include <cyu3os.h>
#include <cyu3error.h>
#include <cyu3gpio.h>
#include <cyu3uart.h>
#include <cyu3usb.h>
#include <cyu3pib.h>
#include <gpif_regs.h>
#include "cyfx_dsc.h"
#include "cyfxgpif2config.h"

enum vendor_command_codes {VENDOR_CONTROL_REQUEST = 0xAA,
                           VENDOR_BULK_REQUEST};

#define CY_FX_APP_THREAD_STACK      (0x1000)
#define CY_FX_APP_THREAD_PRIORITY	(8)
#define CY_FX_FULL_SPEED_EP_SIZE	(64)				/* Sets the end-point size to 64 bytes for full speed */
#define CY_FX_HIGH_SPEED_EP_SIZE	(512)				/* Sets the end-point size to 512 bytes for high speed */
#define CY_FX_SUPER_SPEED_EP_SIZE	(1024)				/* Sets the end-point size to 1024 bytes for SuperSpeed */
#define CY_FX_DEBUG_PRIORITY		(4)					/* Sets the debug print priority level */
#define LED_GPIO					(54)				/* FX3 led */
#define FPGA_RST_GPIO				(33)				/* FX3 DQ16 pin */
#define DEFAULT_BUFFER_SIZE			(16384)				/* DMA channel buffer size */
#define DEFAULT_BUFFER_COUNT		(1)					/* DMA buffers count */
#define DEFAULT_PIB_CLKDIV			(116)				/* Default PIB clkDiv value */

uint8_t Ep0Buffer[512] __attribute__ ((aligned (32)));	/* Local buffer used for vendor command handling. */
//CyU3PDmaMultiChannel DmaChInHandle;
uint8_t SMState;
CyU3PDmaChannel DmaChInHandle;

CyU3PThread AppThread;									/* Application thread structure */

volatile int request_counter = 0;
volatile int gpif_intr_counter = 0;
volatile CyBool_t gpif_fw_triggered = CyFalse;
volatile int glDmaRxCount = 0;
volatile int glDmaTxCount = 0;

/* Application error handler. */
void CyFxAppErrorHandler (CyU3PReturnStatus_t apiRetStatus)
{

	for (;;)
    {
        CyU3PThreadSleep (100);
    }
}

void CyFxIntrCallback(uint8_t stateId)
{
	gpif_intr_counter++;
	CyU3PGpifControlSWInput (CyFalse);
}

void CyFxDmaCallback(
        CyU3PDmaChannel   *chHandle, /* Handle to the DMA channel. */
        CyU3PDmaCbType_t  type,      /* Callback type.             */
        CyU3PDmaCBInput_t *input)    /* Callback status.           */
{
    if (type == CY_U3P_DMA_CB_PROD_EVENT)
    {
        glDmaRxCount++;      // global variable
    }

    if (type == CY_U3P_DMA_CB_CONS_EVENT)
    {
        glDmaTxCount++;    // global variable
    }
}

void EpEvtCallback(CyU3PUsbEpEvtType evType,       /**< Type of event being notified. */
        CyU3PUSBSpeed_t   usbSpeed,     /**< Current USB connection speed. */
        uint8_t           epNum)
{
	CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "EP event: type %d, speed %d, epNum %d\r\n",
			evType, usbSpeed, epNum);
}

/* This function starts the application. This is called
 * when a SET_CONF event is received from the USB host. The endpoints
 * are configured and the DMA pipe is setup in this function. */
void CyFxUSBAppStart (void)
{
	uint16_t size = 0;
    CyU3PEpConfig_t epCfg;
    //CyU3PDmaMultiChannelConfig_t dmaCfg;
    CyU3PDmaChannelConfig_t dmaCfg;
    CyU3PGpioSimpleConfig_t gpioConfig;
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

    CyU3PUSBSpeed_t usbSpeed = CyU3PUsbGetSpeed();

	/* Based on the Bus Speed configure the endpoint packet size */
    switch (usbSpeed)
    {
    case CY_U3P_FULL_SPEED:
        size = CY_FX_FULL_SPEED_EP_SIZE;	/* If connected to full speed, the endpoint size should be 64 bytes */
        CyU3PDebugPrint(CY_FX_DEBUG_PRIORITY,"Connected to Full-Speed host.\r\n");
        break;

    case CY_U3P_HIGH_SPEED:
        size = CY_FX_HIGH_SPEED_EP_SIZE;	/* If connected to high speed, the endpoint size should be 512 bytes */
        CyU3PDebugPrint(CY_FX_DEBUG_PRIORITY,"Connected to High-Speed host.\r\n");
        break;

    case  CY_U3P_SUPER_SPEED:
        size = CY_FX_SUPER_SPEED_EP_SIZE;	/* If connected to Super-speed, the endpoint size should be 1024 bytes */
        CyU3PDebugPrint(CY_FX_DEBUG_PRIORITY,"Connected to Super-Speed host.\r\n");
        break;

    default:
        CyU3PDebugPrint(CY_FX_DEBUG_PRIORITY, "Error! Invalid USB speed.\r\n");
        CyFxAppErrorHandler (CY_U3P_ERROR_FAILURE);
        break;
    }

    /* --- END POINTS --- */
    CyU3PMemSet ((uint8_t *)&epCfg, 0, sizeof (epCfg));
    epCfg.enable   = CyTrue;
    epCfg.epType   = CY_U3P_USB_EP_BULK;
    epCfg.burstLen = 1;
    epCfg.pcktSize = size;

    /* Consumer endpoint configuration */
    apiRetStatus = CyU3PSetEpConfig (CY_FX_EP_CONSUMER, &epCfg);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "CyU3PSetEpConfig failed, Error code = %d\r\n", apiRetStatus);
        CyFxAppErrorHandler (apiRetStatus);
    }

    /* Producer endpoint configuration */
    apiRetStatus = CyU3PSetEpConfig (CY_FX_EP_PRODUCER, &epCfg);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "CyU3PSetEpConfig failed, Error code = %d\r\n", apiRetStatus);
        CyFxAppErrorHandler (apiRetStatus);
    }

    CyU3PUsbFlushEp(CY_FX_EP_CONSUMER);
    CyU3PUsbFlushEp(CY_FX_EP_PRODUCER);

    //CyU3PUsbRegisterEpEvtCallback(EpEvtCallback, 0x1FF, 0x00, 0x02);

    /* --- DMA CONFIG --- */
    dmaCfg.size = DEFAULT_BUFFER_SIZE;
    dmaCfg.count = DEFAULT_BUFFER_COUNT;
    dmaCfg.prodSckId = CY_U3P_PIB_SOCKET_0;
    dmaCfg.consSckId = CY_U3P_UIB_SOCKET_CONS_1;
//    dmaCfg.validSckCount = 2;
//    dmaCfg.prodSckId[0] = CY_U3P_PIB_SOCKET_0;
//    dmaCfg.prodSckId[1] = CY_U3P_PIB_SOCKET_1;
//    dmaCfg.consSckId[0] = CY_U3P_UIB_SOCKET_CONS_1;
    dmaCfg.dmaMode = CY_U3P_DMA_MODE_BYTE;
    dmaCfg.notification = CY_U3P_DMA_CB_PROD_EVENT | CY_U3P_DMA_CB_CONS_EVENT;
    dmaCfg.cb = CyFxDmaCallback;
    dmaCfg.prodHeader = 0;
    dmaCfg.prodFooter = 0;
    dmaCfg.consHeader = 0;
    dmaCfg.prodAvailCount = 0;

    //apiRetStatus = CyU3PDmaMultiChannelCreate (&DmaChInHandle, CY_U3P_DMA_TYPE_AUTO_MANY_TO_ONE, &dmaCfg);
    apiRetStatus = CyU3PDmaChannelCreate (&DmaChInHandle, CY_U3P_DMA_TYPE_AUTO_SIGNAL, &dmaCfg);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "CyU3PDmaChannelCreate failed, Error code = %d\r\n", apiRetStatus);
        CyFxAppErrorHandler (apiRetStatus);
    }

    /* Set DMA Channel transfer size */
    //apiRetStatus = CyU3PDmaMultiChannelSetXfer (&DmaChInHandle, 0, 0);
    apiRetStatus = CyU3PDmaChannelSetXfer (&DmaChInHandle, 0);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "CyU3PDmaChannelSetXfer failed, Error code = %d\r\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* --- GPIF CONFIG --- */
    apiRetStatus = CyU3PGpifLoad (&CyFxGpifConfig);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "CyU3PGpifLoad failed, Error Code=%d\r\n", apiRetStatus);
        CyFxAppErrorHandler (apiRetStatus);
    }

    CyU3PGpifRegisterSMIntrCallback(CyFxIntrCallback);

    apiRetStatus = CyU3PGpifSMStart(IDLE, ALPHA_IDLE);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "CyU3PGpifSMStart failed, Error Code=%d\r\n", apiRetStatus);
        CyFxAppErrorHandler (apiRetStatus);
    }

    /* --- GPIO CONFIG --- */
	gpioConfig.inputEn = CyFalse;
	gpioConfig.outValue = CyTrue; // Default RST level is HIGH, active is LOW
	gpioConfig.driveLowEn = CyTrue;
	gpioConfig.driveHighEn = CyTrue;
	gpioConfig.intrMode = CY_U3P_GPIO_NO_INTR;

	apiRetStatus = CyU3PDeviceGpioOverride (FPGA_RST_GPIO, CyTrue);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint(CY_FX_DEBUG_PRIORITY, "CyU3PDeviceGpioOverride failed, output pin = %d, error code = %d\r\n",
				FPGA_RST_GPIO, apiRetStatus);
		CyFxAppErrorHandler(apiRetStatus);
	}

	apiRetStatus = CyU3PGpioSetSimpleConfig(FPGA_RST_GPIO, &gpioConfig);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		/* Error handling */
		CyU3PDebugPrint(CY_FX_DEBUG_PRIORITY, "CyU3PGpioSetSimpleConfig failed, output pin = %d, error code = %d\r\n",
				FPGA_RST_GPIO, apiRetStatus);
		CyFxAppErrorHandler(apiRetStatus);
	}


}

/* Callback to handle the USB setup requests. */
CyBool_t CyFxUsbSetupCB (
        uint32_t setupdat0, /* SETUP Data 0 */
        uint32_t setupdat1  /* SETUP Data 1 */
    )
{
    /* Fast enumeration is used. Only requests addressed to the interface, class,
     * vendor and unknown control requests are received by this function.
     * This application does not support any class or vendor requests. */
	union {
		uint32_t RawData[2];
		struct {
			uint8_t Target:5;
			uint8_t Type:2;
			uint8_t DirDevToHost:1; // 1 = Device to Host; 0 = Host to Device
			uint8_t Request;
			uint16_t Value;
			uint16_t Index;
			uint16_t Length;
		};
	} SetupRequest;

	SetupRequest.RawData[0] = setupdat0;
	SetupRequest.RawData[1] = setupdat1;

    uint16_t br = 0;
    //uint32_t *arg = (void*)Ep0Buffer;

	CyBool_t isHandled = CyFalse;
	CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

    if (SetupRequest.DirDevToHost
    		&& (SetupRequest.Type == (CY_U3P_USB_VENDOR_RQT >> 5))
    		&& (SetupRequest.Target == CY_U3P_USB_TARGET_INTF))
    {
    	switch (SetupRequest.Value) {

    	// Sending 16 bytes of data TO host
    	case VENDOR_CONTROL_REQUEST:
    		request_counter++;
    		CyU3PUsbSendEP0Data(SetupRequest.Length, Ep0Buffer);
    		isHandled = CyTrue;
    		break;

    	default:
    		CyU3PDebugPrint(CY_FX_DEBUG_PRIORITY, "Unknown vendor code received (%d)\r\n", SetupRequest.Value);
    		CyU3PUsbStall (0, CyTrue, CyFalse);
    	}
    }

    if (!SetupRequest.DirDevToHost
    		&& (SetupRequest.Type == (CY_U3P_USB_VENDOR_RQT >> 5))
    		&& (SetupRequest.Target == CY_U3P_USB_TARGET_INTF))
    {
    	switch (SetupRequest.Value) {

    	// Receiving 16 bytes of data FROM host
    	case VENDOR_CONTROL_REQUEST:
    		request_counter++;
    		CyU3PUsbGetEP0Data(SetupRequest.Length, Ep0Buffer, &br);
    		isHandled = CyTrue;
    		break;

    	case VENDOR_BULK_REQUEST:
    		request_counter++;
    		CyU3PUsbGetEP0Data(SetupRequest.Length, Ep0Buffer, &br);

    		gpif_fw_triggered = CyTrue;
    		CyU3PGpifControlSWInput (CyTrue);

//    	    apiRetStatus = CyU3PGpifSMSwitch(256, MYSTART, 256, ALPHA_MYSTART, 0);
//    	    if (apiRetStatus != CY_U3P_SUCCESS)
//    	    {
//    	    	CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "CyU3PGpifSMSwitch failed, Error Code=%d\r\n", apiRetStatus);
//    	    	CyFxAppErrorHandler (apiRetStatus);
//    	    }

    	    isHandled = CyTrue;
    		break;

    	default:
    		CyU3PDebugPrint(CY_FX_DEBUG_PRIORITY, "Unknown vendor code received (%d)\r\n", SetupRequest.Value);
    		CyU3PUsbStall (0, CyTrue, CyFalse);
    	}
    }

    if (isHandled == CyFalse)
    	CyU3PDebugPrint(CY_FX_DEBUG_PRIORITY, "Unknown EP0 request received\r\n");

    return isHandled;
}

/* This is the callback function to handle the USB events. */
void CyFxUsbEventCB (CyU3PUsbEventType_t evtype, uint16_t evdata)
{
    switch (evtype)
    {
    case CY_U3P_USB_EVENT_CONNECT:
    	CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "Event: CY_U3P_USB_EVENT_CONNECT\r\n");
    	break;

    case CY_U3P_USB_EVENT_DISCONNECT:
        CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "Event: CY_U3P_USB_EVENT_DISCONNECT\r\n");
        break;

    case CY_U3P_USB_EVENT_SUSPEND:
        CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "Event: CY_U3P_USB_EVENT_SUSPEND\r\n");
        break;

    case CY_U3P_USB_EVENT_RESET:
        CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "Event: CY_U3P_USB_EVENT_RESET\r\n");
        break;

    case CY_U3P_USB_EVENT_SETCONF:
    	CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "Event: CY_U3P_USB_EVENT_SETCONF\r\n");
    	CyFxUSBAppStart();
    	break;

    case CY_U3P_USB_EVENT_SPEED:
        CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "Event: CY_U3P_USB_EVENT_SPEED\r\n");
        break;

    case CY_U3P_USB_EVENT_VBUS_VALID:
        CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "Event: CY_U3P_USB_EVENT_VBUS_VALID\r\n");
        break;

    case CY_U3P_USB_EVENT_SET_SEL:
        CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "Event: CY_U3P_USB_EVENT_SET_SEL\r\n");
        break;

    case CY_U3P_USB_EVENT_EP0_STAT_CPLT:
    	break;

    default:
    	CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "Event: UNKNOWN (%d)\r\n", evtype);
    }
}

CyBool_t CyFxUsbLpmCB (CyU3PUsbLinkPowerMode link_mode)
{
    return CyTrue;
}

void PibEventCallback (CyU3PPibIntrType cbType, uint16_t cbArg)
{
    if (cbType == CYU3P_PIB_INTR_ERROR)
    {
        switch (CYU3P_GET_PIB_ERROR_TYPE (cbArg))
        {
            case CYU3P_PIB_ERR_THR0_WR_OVERRUN:
                CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "CYU3P_PIB_ERR_THR0_WR_OVERRUN\r\n");
                break;
            case CYU3P_PIB_ERR_THR1_WR_OVERRUN:
                CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "CYU3P_PIB_ERR_THR1_WR_OVERRUN\r\n");
                break;
            case CYU3P_PIB_ERR_THR2_WR_OVERRUN:
                CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "CYU3P_PIB_ERR_THR2_WR_OVERRUN\r\n");
                break;
            case CYU3P_PIB_ERR_THR3_WR_OVERRUN:
                CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "CYU3P_PIB_ERR_THR3_WR_OVERRUN\r\n");
                break;
            case CYU3P_PIB_ERR_THR0_RD_UNDERRUN:
                CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "CYU3P_PIB_ERR_THR0_RD_UNDERRUN\r\n");
                break;
            case CYU3P_PIB_ERR_THR1_RD_UNDERRUN:
                CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "CYU3P_PIB_ERR_THR1_RD_UNDERRUN\r\n");
                break;
            case CYU3P_PIB_ERR_THR2_RD_UNDERRUN:
                CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "CYU3P_PIB_ERR_THR2_RD_UNDERRUN\r\n");
                break;
            case CYU3P_PIB_ERR_THR3_RD_UNDERRUN:
                CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "CYU3P_PIB_ERR_THR3_RD_UNDERRUN\r\n");
                break;
            case CYU3P_PIB_ERR_NONE:
                //CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "CYU3P_PIB_ERR_NONE\r\n");
                break;
            default:
                CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "Unknown PIB error code %d\r\n", CYU3P_GET_PIB_ERROR_TYPE (cbArg));
                break;
        }
    }
}

/* Initialize the debug module with UART. */
void CyFxApplnDebugInit (void)
{
    CyU3PUartConfig_t uartConfig;
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

    /* Initialize and configure the UART for logging. */
    apiRetStatus = CyU3PUartInit ();
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
    	CyFxAppErrorHandler (apiRetStatus);
    }

    CyU3PMemSet ((uint8_t *)&uartConfig, 0, sizeof (uartConfig));
    uartConfig.baudRate = CY_U3P_UART_BAUDRATE_115200;
    uartConfig.stopBit  = CY_U3P_UART_ONE_STOP_BIT;
    uartConfig.parity   = CY_U3P_UART_NO_PARITY;
    uartConfig.txEnable = CyTrue;
    uartConfig.rxEnable = CyFalse;
    uartConfig.flowCtrl = CyFalse;
    uartConfig.isDma    = CyTrue;
    apiRetStatus = CyU3PUartSetConfig (&uartConfig, NULL);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
    	CyFxAppErrorHandler (apiRetStatus);
    }

    /* Set the dma for an inifinity transfer */
    apiRetStatus = CyU3PUartTxSetBlockXfer (0xFFFFFFFF);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
    	CyFxAppErrorHandler (apiRetStatus);
    }

    /* Start the debug module for printing log messages */
    apiRetStatus = CyU3PDebugInit (CY_U3P_LPP_SOCKET_UART_CONS, 8);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
    	CyFxAppErrorHandler (apiRetStatus);
    }

    CyU3PDebugPreamble(CyFalse);
}

void CyFxAppGpioInit (void)
{
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
	CyU3PGpioSimpleConfig_t gpioConfig;
    CyU3PGpioClock_t gpioClock;

    gpioClock.fastClkDiv = 2;
	gpioClock.slowClkDiv = 64;
	gpioClock.simpleDiv  = CY_U3P_GPIO_SIMPLE_DIV_BY_2;
	gpioClock.clkSrc     = CY_U3P_SYS_CLK;
	gpioClock.halfDiv    = 0;
	apiRetStatus = CyU3PGpioInit (&gpioClock, NULL);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		/* Error Handling */
		CyU3PDebugPrint(CY_FX_DEBUG_PRIORITY, "CyU3PGpioInit failed, error code = %d\r\n",
				apiRetStatus);
		CyFxAppErrorHandler(apiRetStatus);
	}

	/* LED GPIO */
	apiRetStatus = CyU3PDeviceGpioOverride (LED_GPIO, CyTrue);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		/* Error Handling */
		CyU3PDebugPrint(CY_FX_DEBUG_PRIORITY, "CyU3PDeviceGpioOverride failed, error code = %d\r\n",
				apiRetStatus);
		CyFxAppErrorHandler(apiRetStatus);
	}

	CyU3PMemSet ((uint8_t *)&gpioConfig, 0, sizeof (gpioConfig));
	gpioConfig.outValue = CyTrue;
	gpioConfig.driveLowEn = CyTrue;
	gpioConfig.driveHighEn = CyTrue;
	gpioConfig.inputEn = CyFalse;
	gpioConfig.intrMode = CY_U3P_GPIO_NO_INTR;
	apiRetStatus = CyU3PGpioSetSimpleConfig(LED_GPIO, &gpioConfig);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		/* Error handling */
		CyU3PDebugPrint(CY_FX_DEBUG_PRIORITY, "CyU3PGpioSetSimpleConfig failed, error code = %d\r\n",
				apiRetStatus);
		CyFxAppErrorHandler(apiRetStatus);
	}

}

void CyFxApplnInit (void)
{
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    CyU3PPibClock_t pibClock;

	/* Initialize the p-port block. We are running the PIB Block at 48 MHz. DLL needs to be enabled for Master
     * operation of GPIF. */
    pibClock.clkDiv      = DEFAULT_PIB_CLKDIV;
    pibClock.clkSrc      = CY_U3P_SYS_CLK;
    pibClock.isHalfDiv   = CyFalse;
    pibClock.isDllEnable = CyFalse;

    apiRetStatus = CyU3PPibInit (CyTrue, &pibClock);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "P-port Initialization failed, Error Code=%d\r\n", apiRetStatus);
        CyFxAppErrorHandler (apiRetStatus);
    }

    CyU3PPibRegisterCallback(PibEventCallback, 0xffff); // 0xffff is event mask (all events)

    /* Start the USB functionality. */
    apiRetStatus = CyU3PUsbStart();
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(CY_FX_DEBUG_PRIORITY, "CyU3PUsbStart failed to Start, Error code = %d\r\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* The fast enumeration is the easiest way to setup a USB connection,
     * where all enumeration phase is handled by the library. Only the
     * class / vendor requests need to be handled by the application. */
    CyU3PUsbRegisterSetupCallback(CyFxUsbSetupCB, CyTrue);

    /* Setup the callback to handle the USB events. */
    CyU3PUsbRegisterEventCallback(CyFxUsbEventCB);

    /* Register a callback to handle LPM requests from the USB 3.0 host. */
    CyU3PUsbRegisterLPMRequestCallback(CyFxUsbLpmCB);

    /* Set the USB Enumeration descriptors */

    /* Super speed device descriptor. */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_SS_DEVICE_DESCR, 0, (uint8_t *)CyFxUSB30DeviceDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(CY_FX_DEBUG_PRIORITY, "USB set device descriptor failed, Error code = %d\r\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* High speed device descriptor. */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_HS_DEVICE_DESCR, 0, (uint8_t *)CyFxUSB20DeviceDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(CY_FX_DEBUG_PRIORITY, "USB set device descriptor failed, Error code = %d\r\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* BOS descriptor */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_SS_BOS_DESCR, 0, (uint8_t *)CyFxUSBBOSDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(CY_FX_DEBUG_PRIORITY, "USB set configuration descriptor failed, Error code = %d\r\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Device qualifier descriptor */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_DEVQUAL_DESCR, 0, (uint8_t *)CyFxUSBDeviceQualDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(CY_FX_DEBUG_PRIORITY, "USB set device qualifier descriptor failed, Error code = %d\r\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Super speed configuration descriptor */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_SS_CONFIG_DESCR, 0, (uint8_t *)CyFxUSBSSConfigDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(CY_FX_DEBUG_PRIORITY, "USB set configuration descriptor failed, Error code = %d\r\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* High speed configuration descriptor */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_HS_CONFIG_DESCR, 0, (uint8_t *)CyFxUSBHSConfigDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(CY_FX_DEBUG_PRIORITY, "USB Set Other Speed Descriptor failed, Error Code = %d\r\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Full speed configuration descriptor */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_FS_CONFIG_DESCR, 0, (uint8_t *)CyFxUSBFSConfigDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(CY_FX_DEBUG_PRIORITY, "USB Set Configuration Descriptor failed, Error Code = %d\r\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* String descriptor 0 */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_STRING_DESCR, 0, (uint8_t *)CyFxUSBStringLangIDDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(CY_FX_DEBUG_PRIORITY, "USB set string descriptor failed, Error code = %d\r\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* String descriptor 1 */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_STRING_DESCR, 1, (uint8_t *)CyFxUSBManufactureDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(CY_FX_DEBUG_PRIORITY, "USB set string descriptor failed, Error code = %d\r\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* String descriptor 2 */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_STRING_DESCR, 2, (uint8_t *)CyFxUSBProductDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(CY_FX_DEBUG_PRIORITY, "USB set string descriptor failed, Error code = %d\r\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* String descriptor 3 */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_STRING_DESCR, 3, (uint8_t *)CyFxUsbSNDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(CY_FX_DEBUG_PRIORITY, "USB set string descriptor failed, Error code = %d\r\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Connect the USB Pins with super speed operation enabled. */
    apiRetStatus = CyU3PConnectState(CyTrue, CyTrue);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint(CY_FX_DEBUG_PRIORITY, "USB Connect failed, Error code = %d\r\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }

    CyU3PDebugPrint(CY_FX_DEBUG_PRIORITY, "Initialization completed.\r\n");
}

/* Entry function for the gpioOutputThread */
void AppThread_Entry (uint32_t input)
{
    /* Initialize the debug module */
    CyFxApplnDebugInit();

    /* Initialize the gpio LED output */
    CyFxAppGpioInit();

    /* Initialize the bulk loop application */
    CyFxApplnInit();

    for (;;)
    {
    	CyU3PGpioSetValue(LED_GPIO, CyFalse);
        CyU3PThreadSleep (250);
    	CyU3PGpioSetValue(LED_GPIO, CyTrue);
        CyU3PThreadSleep (750);
//    	CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "EP0 requests count (%d), interrupt count (%d)\r\n",
//    			request_counter, gpif_intr_counter);
        CyU3PGpifGetSMState(&SMState);
    	CyU3PDebugPrint (CY_FX_DEBUG_PRIORITY, "glDmaRxCount (%d), glDmaTxCount (%d), GPIF state = %d, INTr CPU = %d, request = %d\r\n",
    			glDmaRxCount, glDmaTxCount, SMState,gpif_intr_counter,request_counter);

    }
}

/* Application define function which creates the threads. */
void CyFxApplicationDefine (void)
{
	void *ptr = NULL;
    uint32_t retThrdCreate = CY_U3P_SUCCESS;

    /* Allocate the memory for the threads */
    ptr = CyU3PMemAlloc (CY_FX_APP_THREAD_STACK);

    if (ptr != NULL) {
    	/* Create the thread for the application */
    	retThrdCreate = CyU3PThreadCreate (&AppThread,	/* Application thread structure */
    			"21:Main thread",               		/* Thread ID and Thread name */
    			AppThread_Entry,                		/* Application thread entry function */
    			0,                                     	/* No input parameter to thread */
    			ptr,                                   	/* Pointer to the allocated thread stack */
    			CY_FX_APP_THREAD_STACK,            		/* Thread stack size */
    			CY_FX_APP_THREAD_PRIORITY,         		/* Thread priority */
    			CY_FX_APP_THREAD_PRIORITY,         		/* Pre-emption threshold for the thread. */
    			CYU3P_NO_TIME_SLICE,					/* No time slice for the application thread */
    			CYU3P_AUTO_START						/* Start the thread immediately */
    	);
    }

    /* Check the return code */
    if (retThrdCreate != 0)
    {
        /* Thread creation failed with the error code retThrdCreate */

        /* Add custom recovery or debug actions here */

        /* Application cannot continue */
        /* Loop indefinitely */
        while(1);
    }



}

/*
 * Main function
 */
int main (void)
{
    CyU3PIoMatrixConfig_t io_cfg;
    CyU3PSysClockConfig_t clk_cfg;
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

    /* setSysClk400 clock configurations */
    /* SYS_CLK may be 384 MHz or 403.2 MHz depending upon whether the 'setSysClk400' field */
    clk_cfg.setSysClk400  = CyFalse;   /* FX3 device's master clock is set to a frequency > 400 MHz */
    clk_cfg.cpuClkDiv     = 2;        /* CPU clock divider */
    clk_cfg.dmaClkDiv     = 2;        /* DMA clock divider */
    clk_cfg.mmioClkDiv    = 2;        /* MMIO clock divider */
    clk_cfg.useStandbyClk = CyFalse;  /* device has no 32KHz clock supplied */
    clk_cfg.clkSrc = CY_U3P_SYS_CLK;  /* Clock source for a peripheral block  */

    /* Initialize the device */
    apiRetStatus = CyU3PDeviceInit (&clk_cfg);
    if (apiRetStatus != CY_U3P_SUCCESS)
        goto handle_fatal_error;

    /* Initialize the caches. Enable both Instruction and Data Caches. */
    apiRetStatus = CyU3PDeviceCacheControl (CyTrue, CyTrue, CyTrue);
    if (apiRetStatus != CY_U3P_SUCCESS)
    	goto handle_fatal_error;

    /* Configure the IO matrix for the device. On the FX3 DVK board, the COM port
     * is connected to the IO(53:56). This means that either DQ32 mode should be
     * selected or lppMode should be set to UART_ONLY. */
    CyU3PMemSet ((uint8_t *)&io_cfg, 0, sizeof(io_cfg));
    io_cfg.useUart = CyTrue;
    io_cfg.lppMode = CY_U3P_IO_MATRIX_LPP_UART_ONLY;


    apiRetStatus = CyU3PDeviceConfigureIOMatrix (&io_cfg);
    if (apiRetStatus != CY_U3P_SUCCESS)
        goto handle_fatal_error;

    /* This is a non returnable call for initializing the RTOS kernel */
    CyU3PKernelEntry ();

    /* Dummy return to make the compiler happy */
    return 0;

handle_fatal_error:
    /* Cannot recover from this error. */
    while (1);
}
